package example.localplugin;

import gridsim.Gridlet;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.Random;

import schedframe.scheduling.TaskInterface;
import schedframe.scheduling.events.SchedulingEvent;
import schedframe.scheduling.plugin.grid.ModuleList;
import test.rewolucja.GSSIMJobInterface;
import test.rewolucja.energy.profile.PStateType;
import test.rewolucja.resources.manager.implementation.ClusterResourceManager;
import test.rewolucja.resources.manager.interfaces.ResourceManagerInterface;
import test.rewolucja.resources.physical.implementation.ComputingNode;
import test.rewolucja.resources.physical.implementation.Processor;
import test.rewolucja.scheduling.JobRegistryInterface;
import test.rewolucja.scheduling.plan.SchedulingPlanInterfaceNew;
import test.rewolucja.scheduling.plan.SchedulingPlanNew;
import test.rewolucja.scheduling.queue.Queue;
import test.rewolucja.scheduling.queue.QueueList;

public class FCFSRandomClusterLocalPlugin extends BaseLocalPlugin {

	private Random rand;
	private boolean init = true;
	
	public FCFSRandomClusterLocalPlugin() {
		rand = new Random(5);
	}

	public SchedulingPlanInterfaceNew schedule(SchedulingEvent event, QueueList queues, JobRegistryInterface jobRegistry,
			 ResourceManagerInterface resManager, ModuleList modules) {
		
		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		
		if( init)
		{
			List<Processor> cpus = resourceManager.getProcessors();
			
			for( Processor cpu : cpus)
				cpu.getPowerInterface().setPState( PStateType.P0);
			
			init = false;
		}
		

		
		SchedulingPlanNew plan = new SchedulingPlanNew();
		// chose the events types to serve.
		// Different actions for different events are possible.
		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:
			// our tasks are placed only in first queue (see
			// BaseLocalPlugin.placeJobsInQueues() method)
			Queue q = queues.get(0);
			// check all tasks in queue

			for (int i = 0; i < q.size(); i++) {
				GSSIMJobInterface<?> job = q.get(i);
				TaskInterface<?> task = (TaskInterface<?>) job;
				// if status of the tasks in READY
				if (task.getStatus() == Gridlet.READY) {
					String nodeName = chooseRandomProvider(resourceManager, task);
					if (nodeName != null) {
						addToSchedulingPlan(plan, task, nodeName);
					}
				}
			}
			break;
		}
		return plan;
	}

	private List<ComputingNode> findSuitableNodes(int cpuRequest, List<ComputingNode> nodes){
		List<ComputingNode> avNodes = new ArrayList<ComputingNode>();
		for(ComputingNode node: nodes){
			if(node.getFreeProcessorsNumber() >= cpuRequest){
				avNodes.add(node);
			}
		}
		return avNodes;
	}
	
	private String chooseRandomProvider(ClusterResourceManager resourceManager, TaskInterface<?> task) {
		
		int cpuRequest;
		try {
			cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
		} catch (NoSuchFieldException e) {
			cpuRequest = 1;
		}
		
		List<ComputingNode> nodes = resourceManager.getComputingNodes();
		
		nodes = findSuitableNodes(cpuRequest, nodes);
		
		if( nodes.size() > 0)
		{	
			int nodeIdx = rand.nextInt(nodes.size());
			return nodes.get(nodeIdx).getName();
		}
		else
			return null;
	}
	
	public String getName() {
		return getClass().getName();
	}

	public void init(Properties properties) {
		// no extra initialization is expected.
	}

}

